/*
 * main.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#define SOCKET_DEFAULT	"/.gtk-notify.socket"

#define WRITE(a,b,c) {if(write(a,b,c) == -1){perror(argv[0]);}}

int main (int argc, char *argv[]) {
	int c, verbose = 0;
	char action = 0;
	char *text = NULL, *icon = NULL, *cmd = NULL, *socket_name = NULL;
	int sock_fd, result;
	struct sockaddr_un address;

	while ((c = getopt(argc, argv, "APCs:t:i:c:vh")) != -1) {
		switch (c) {
			case 'A':
			case 'P':
			case 'C':
				if (!action) {
					action = c;
				} else {
					fprintf(stderr, "%s: Only one action can be specified at once.\n", argv[0]);
					exit(EXIT_FAILURE);
				}
				break;
			case 's':
				socket_name = strdup(optarg);
				break;
			case 't':
				text = strdup(optarg);
				break;
				
			case 'i':
				icon = strdup(optarg);
				break;
				
			case 'c':
				cmd = strdup(optarg);
				break;
				
			case 'v':
				verbose = 1;
				break;
				
			default:
			case '*':
			case 'h':
				fprintf(stderr, "Usage: %s ( -A | -P | -C ) -s socket_filename -t text "
						"-i icon.png [-c shell_command]\n\n"
						"\t-A  ...  Append event (require -s, -t, -i)\n"
						"\t-P  ...  Pickup event\n"
						"\t-C  ...  Open config dialog\n\n",
						argv[0]);
				exit(EXIT_FAILURE);
		}	
	}
	
	if (!socket_name) {
		socket_name = (char*)malloc(strlen(getenv("HOME") + strlen(SOCKET_DEFAULT) + 2));
		strcpy(socket_name, getenv("HOME"));
		strcat(socket_name, SOCKET_DEFAULT);
	}

	if (!action) {
		fprintf(stderr, "Please, specify action.\n");
		exit(EXIT_FAILURE);
	}

	if (verbose) {
		printf( "Action:   %c\n"
			"Socket:   %s\n"
			"Text:     %s\n"
			"Icon:     %s\n"
			"Command:  %s\n",
			action ? action : '-',
			socket_name ? socket_name : "(none)",
			text ? text : "(none)",
			icon ? icon : "(none)",
			cmd ? cmd : "(none)");
	}

	if (action == 'A' && (!socket_name || !text || !icon)) {
		fprintf(stderr, "Missing arguments!\n");
		exit(EXIT_FAILURE);
	}
	
	if (verbose)
		fprintf(stderr, "%s: Connecting to socket...\n", argv[0]);
	sock_fd = socket(AF_UNIX, SOCK_STREAM, 0);
	address.sun_family = AF_UNIX;
	strcpy(address.sun_path, socket_name);
	result = connect(sock_fd, (struct sockaddr *)&address, sizeof(address));
	if (result == -1) {
		perror(argv[0]);
		exit(EXIT_FAILURE);
	}

	if (verbose)
		fprintf(stderr, "%s: Sending data...\n", argv[0]);
	
	WRITE(sock_fd, &action, 1);
	WRITE(sock_fd, "\0", 1);
	if (action == 'A') {
		WRITE(sock_fd, text, strlen(text) + 1);
		WRITE(sock_fd, icon, strlen(icon) + 1);
		if (cmd)
			write(sock_fd, cmd, strlen(cmd) + 1);
	}
	if (close(sock_fd)) {
		perror(argv[0]);
		exit(EXIT_FAILURE);
	}
	
	if (verbose)
		fprintf(stderr, "%s: Done.\n", argv[0]);

	return EXIT_SUCCESS;
}
