/*
 * events.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include "interface.h"
#include "support.h"
#include "events.h"
#include "notify.h"

#define DEBUG 0
#include "debug.h"

/** Fronta udalosti */
struct s_event_queue {
	/* list */
	struct s_event_queue *next, *prev;
	
	/* data */
	char *text, *icon, *cmd;
	GtkWidget *icon_widget;
};

struct s_event_queue *event_queue = NULL,	/* nejstarsi udalost - zacatek seznamu */
		     *event_queue_end = NULL;	/* nejnovejsi udalost - konec seznamu */


#if DEBUG >= 6
void event_dump_all (void) {
	struct s_event_queue *pos;
	DBG_MSG(6, "begin: %9p", event_queue);
	for (pos = event_queue; pos; pos = pos->next) {
		DBG_MSG(6, "%9p: prev = %9p, next = %9p, text = \"%s\"",
				pos, pos->prev, pos->next, pos->text);
	}
	DBG_MSG(6, "end:   %9p", event_queue_end);
}
#else
#	define event_dump_all()
#endif


/** Vyzvednuti udalosti z fronty (zrusi to ikonku)
 */
void event_pickup (gpointer p_item) {
	struct s_event_queue *item = (struct s_event_queue *)p_item;
	
	DBG_MSG(5, "event_pickup: %p", item);
	
	if (!item)
		item = event_queue;
	if (item) {
		/* Execute command */
		if (item->cmd) {
			if (fork() == 0) {
				exit(system(item->cmd));
			}
		}

		/* Remove item from list */
		if(item->prev) {
			item->prev->next = item->next;
		} else {
			event_queue = item->next;
			if (event_queue)
				event_queue->prev = NULL;
		}
		if (item->next) {
			item->next->prev = item->prev;
		} else {
			event_queue_end = item->prev;
			if (event_queue_end)
				event_queue_end->next = NULL;
		}
		
		/* destroy item */
		if (item->icon_widget)
			notify_remove_icon(item->icon_widget);
		if (item->text)
			free(item->text);
		if (item->icon)
			free(item->icon);
		if (item->cmd)
			free(item->cmd);
		free(item);
	}
	event_dump_all();
}

/** Pridani udalosti do fronty (vytvori ikonku)
 */
int event_append(char *text, char *icon, char *cmd) {
	struct s_event_queue *new_item = (struct s_event_queue *)malloc(sizeof(struct s_event_queue));

	DBG_MSG(5, "event_append: %p", new_item);
	
	/* nastaveni zacatku seznamu */
	if (event_queue && event_queue_end) {
		new_item->prev = event_queue_end;
		new_item->next = NULL;
		event_queue_end->next = new_item;
		event_queue_end = new_item;
	} else {
		new_item->prev = NULL;
		new_item->next = NULL;
		event_queue = new_item;
		event_queue_end = new_item;
	}
	
	new_item->text = text ? strdup(text) : NULL;
	new_item->icon = icon ? strdup(icon) : NULL;
	new_item->cmd = cmd ? strdup(cmd) : NULL;
	new_item->icon_widget = notify_append_icon(new_item->icon, new_item);
	event_queue_end = new_item;
	
	event_dump_all();
	return 1;
}

/** Zobrazeni cfg. dialogu
 */
void event_show_cfg (void) {
	DBG_MSG(5, "event_show_cfg");
	GtkWidget *config_dialog = NULL;
	config_dialog = create_config_dialog ();
	gtk_widget_show (config_dialog);
}

