/*
 * options.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include "options.h"

struct s_opt opt = {
	/* char *unix_sock_file; */ NULL,
	/* int unix_sock_perm;  */ 0600,
	/* int show_config; */ 0,
	/* int animations; */ 1,
	/* int anim_show; */ 1,
	/* int anim_hide; */ 1,
	/* int anim_step; */ 2,
	/* int anim_delay; */ 30
};

int options_parse(int argc, char **argv) {
	int option_index = 0;
	int c, i;
	char *tmp;
	struct option long_options[] = {
		{ "unix-socket-file", required_argument, 0, 0 },
		{ "unix-socket-perm", required_argument, 0, 0 },
		{ "show-config", no_argument, 0, 'c' },
		{ "no-anim", no_argument, 0, 'a' },
		{ "no-anim-show", no_argument, 0, 0 },
		{ "no-anim-hide", no_argument, 0, 0 },
		{ "anim-step", required_argument, 0, 0 },
		{ "anim-delay", required_argument, 0, 0 },
		{ "help", no_argument, 0, 0 },
		{ 0, 0, 0, 0}
	};
	enum {
		unix_socket_file,
		unix_socket_perm,
		show_config,
		no_anim,
		no_anim_show,
		no_anim_hide,
		anim_step,
		anim_delay,
		help
	};
	
	/* parse */
	while ((c = getopt_long_only(argc, argv, "ac", long_options, &option_index)) != -1) {
		switch (c) {
			case 'c': /* show-config */
				opt.show_config = 1;
				break;
			case 'a': /* no-anim */
				opt.animations = 0;
				break;
			case 0:
				switch (option_index) {
					case unix_socket_file:
						opt.unix_sock_file = strdup(optarg);
						break;
					case unix_socket_perm:
						opt.unix_sock_perm = atoi(optarg);
						break;
					case no_anim_show:
						opt.anim_show = 0;
						break;
					case no_anim_hide:
						opt.anim_hide = 0;
						break;
					case anim_step:
						opt.anim_step = atoi(optarg);
						break;
					case anim_delay:
						opt.anim_delay = atoi(optarg);
						break;
					case help:
						printf("Usage: gtk-notify ");
						for (i = 0; long_options[i].name; i++)
							printf("--%s%s", long_options[i].name, (i + 1) % 3 ? " " : "\n\t\t");
						printf("\n");
						exit(EXIT_SUCCESS);
						break;
				}
				break;
			case '?':
				/* getopt_long already printed an error message. */
				exit(-1);
			default:
				abort ();
		}
	}

	/* init defaults */
	if (!opt.unix_sock_file)
		opt.unix_sock_file = strdup("~/.gtk-notify.socket");
	
	/* apply s/^~\//$HOME\// */
	if (opt.unix_sock_file && opt.unix_sock_file[0] == '~' && opt.unix_sock_file[1] == '/') {
		tmp = (char *) malloc(strlen(opt.unix_sock_file ? opt.unix_sock_file : "") + strlen(getenv("HOME") + 2));
		strcpy(tmp, getenv("HOME"));
		strcat(tmp, opt.unix_sock_file + 1);
		free(opt.unix_sock_file);
		opt.unix_sock_file = tmp;
	}

	if (opt.show_config) {
		fprintf(stderr, "unix-socket-file = \"%s\"\n"
				"unix-socket-perm = \"%03o\"\n"
				"show-config = \"%d\"\n"
				"no-anim = \"%d\"\n"
				"no-anim-show = \"%d\"\n"
				"no-anim-hide = \"%d\"\n"
				"anim-step = \"%d\"\n"
				"anim-delay = \"%d\"\n",
				opt.unix_sock_file,
				opt.unix_sock_perm,
				opt.show_config,
				opt.animations,
				opt.anim_show,
				opt.anim_hide,
				opt.anim_step,
				opt.anim_delay);
	}
}

