/*
 * socket.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */

/* Description:
 *
 * Comands:
 *	Pickup	- vyzvednuti nejstarsi udalosti
 *	Add	- pridani udalosti (text, ikona, prikaz)
 *	Config	- otevreni konfiguracniho dialogu
 *	
 */
#define USE_SOCKET  1

#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include "socket.h"
#include "events.h"
#include "options.h"

#define DEBUG 0
#include "debug.h"

#if USE_SOCKET == 1
#define MAX_RD_DATA_LEN	2048	/* TODO remove this */

static int socket_fd = -1;
static GIOChannel *channel;
static guint channel_tag;
#endif

gboolean socket_new_client (GIOChannel *source, GIOCondition condition, gpointer data);


int socket_init(void) {
#if USE_SOCKET == 1
	struct sockaddr_un server_addr;
	
	/* init socketu */
	unlink(opt.unix_sock_file);
	socket_fd = socket(AF_UNIX, SOCK_STREAM, 0);
	server_addr.sun_family = AF_UNIX;
	strcpy(server_addr.sun_path, opt.unix_sock_file);
	bind(socket_fd, (struct sockaddr *)&server_addr, sizeof(server_addr));
	listen(socket_fd, 5);

	/* registrace callbacku */
	channel = g_io_channel_unix_new(socket_fd);
	channel_tag = g_io_add_watch(channel, G_IO_IN, socket_new_client, NULL);
	
	return 0;
#else
	return 1;
#endif
}


int socket_cleanup(void) {
#if USE_SOCKET == 1
	return 0;
#else
	return 1;
#endif
}

gboolean socket_new_client (GIOChannel *source, GIOCondition condition, gpointer data) {
#if USE_SOCKET == 1
	struct sockaddr_un client_addr;
	int client_len = 0, client_fd = 0, rd_data_len = 0;
	char rd_data[MAX_RD_DATA_LEN];
	char action, *text = NULL, *icon = NULL, *cmd = NULL;
	char **ptr[] = { &text, &icon, &cmd, NULL }, *p;
	int i;

	switch (condition) {
		case G_IO_IN:
			/* create connection with client */
			client_len = sizeof(client_addr);
			if ((client_fd = accept(socket_fd, (struct sockaddr *)&client_addr, &client_len)) == -1) {
				perror("accept");
				break;
			}
			
			/* load all data to buffer */
			rd_data_len = 0;
			bzero(rd_data, MAX_RD_DATA_LEN);
			while ((i = read(client_fd, rd_data + rd_data_len, MAX_RD_DATA_LEN - rd_data_len - 1)) > 0) {
				DBG_MSG(8, "Received %d bytes: \"%s\"", i, rd_data + rd_data_len);
				rd_data_len += i;
			}
			DBG_MSG(8, "Total: Received %d bytes: \"%s\"", rd_data_len, rd_data);
			
			/* terminate data */
			rd_data[rd_data_len] = 0;

			/* if some data received */
			if (rd_data_len > 0) {
				action = rd_data[0];
				switch (action) {
					case 'A':
						text = rd_data + 2;
						icon = text + strlen(text) + 1;
						cmd = icon + strlen(icon) + 1;
						DBG_MSG(8, "text = \"%s\", icon = \"%s\", cmd  = \"%s\"",
								text, icon, cmd);
						event_append(text, icon, cmd);
						break;
					case 'P':
						event_pickup(NULL);
						break;
					case 'C':
						event_show_cfg();
						break;
					default:
						fprintf(stderr, "Your client is too stupid!\n");
				}
			}

			/* say bye bye */
			close(client_fd);
			break;
		default:
			fprintf(stderr, "Opps.\n");
			exit(-1);
	}
	return TRUE;
#else
	return FALSE;
#endif
}
